@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: Subject.cpp 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: Subject.cpp 1691 2008-04-25 16:37:20Z mghie $
 
 */
 
@@ -81,8 +81,12 @@ void Subject::detachObserver(Observer* observer)
 //-----------------------------------------------------------------------------
 void Subject::detachAllObservers()
 {
-    for (ObserverIterator i = observersM.begin(); i != observersM.end(); ++i)
-        (*i)->removeSubject(this);
+    for (ObserverIterator i = observersM.begin(); i != observersM.end();)
+    {
+        Observer* o = *i;   // move the iterator to a 'safe' place because
+        ++i;                // some observers might remove themselves
+        o->removeSubject(this);
+    }
     observersM.clear();
 }
 //-----------------------------------------------------------------------------
@@ -94,8 +98,12 @@ void Subject::notifyObservers()
     {
         // make sure there are no reentrancy problems
         ++locksCountM;
-        for (ObserverIterator i = observersM.begin(); i != observersM.end(); ++i)
-            (*i)->doUpdate();
+        for (ObserverIterator i = observersM.begin(); i != observersM.end();)
+        {
+            Observer* o = *i;   // move the iterator to a 'safe' place because
+            ++i;                // some observers might remove themselves
+            o->doUpdate();
+        }
         --locksCountM;
         needsNotifyObjectsM = false;
     }
