@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: DataGeneratorFrame.cpp 1585 2007-12-10 09:56:09Z mghie $
+  $Id: DataGeneratorFrame.cpp 1736 2008-09-12 22:47:37Z babuskov $
 
 */
 
@@ -527,7 +527,7 @@ wxString getDefaultRange(Domain *d)
     if (dt == wxT("Smallint") || dt == wxT("Float"))
         return wxT("0-100");
 
-    if (   dt == wxT("Numeric") || dt == wxT("Integer")
+    if (   dt == wxT("Numeric") || dt == wxT("Integer") || dt == wxT("Bigint")
         || dt == wxT("Decimal") || dt == wxT("Double precision"))
     {
         return wxT("0-2000000");
