@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: ExecuteSqlFrame.h 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: ExecuteSqlFrame.h 1733 2008-08-29 20:41:31Z mghie $
 
 */
 
@@ -41,7 +41,6 @@
 #include "core/StringUtils.h"
 #include "gui/BaseFrame.h"
 #include "gui/FindDialog.h"
-#include "logger.h"
 #include "sql/SqlStatement.h"
 #include "metadata/database.h"
 #include "statementHistory.h"
@@ -56,6 +55,7 @@ private:
 public:
     SqlEditor(wxWindow *parent, wxWindowID id);
     void markText(int start, int end);
+    void setChars(bool firebirdIdentifierOnly);
     void setFont();
 
     void OnContextMenu(wxContextMenuEvent& event);
@@ -68,13 +68,9 @@ class ExecuteSqlFrame: public BaseFrame, public Observer
 public:
     Database *getDatabase();
     void showProperties(wxString objectName);
-    // query parsing and execution
+
     void executeAllStatements(bool autoExecute = false);
-    void prepareAndExecute(bool prepareOnly = false);
-    bool parseStatements(const wxString& statements, bool autoExecute = false,
-        bool prepareOnly = false, int selectionOffset = 0);
-    bool execute(wxString sql, const wxString& terminator,
-        bool prepareOnly = false);
+
     bool loadSqlFile(const wxString& filename);
     void setSql(wxString sql);
     void clearStats();
@@ -92,12 +88,19 @@ public:
     };
 
 private:
+    // query parsing and execution
+    void prepareAndExecute(bool prepareOnly = false);
+    bool parseStatements(const wxString& statements, bool autoExecute = false,
+        bool prepareOnly = false, int selectionOffset = 0);
+    bool execute(wxString sql, const wxString& terminator,
+        bool prepareOnly = false);
+
     std::vector<SqlStatement> executedStatementsM;
     wxString filenameM;
 
     typedef enum { ttNormal, ttSql, ttError } TextType;
     void log(wxString s, TextType type = ttNormal);     // write messages to textbox
-    void SplitScreen();
+    void splitScreen();
     Database* databaseM;
     DatabaseToSystemCharsetConversion dbCharsetConversionM;
     void setDatabase(Database* db);
@@ -109,19 +112,21 @@ private:
     bool inTransactionM;
     IBPP::Transaction transactionM;
     IBPP::Statement statementM;
-    void InTransaction(bool started);       // changes controls (enable/disable)
+    void inTransaction(bool started);       // changes controls (enable/disable)
     bool commitTransaction();
-    void rollbackTransaction();
+    bool rollbackTransaction();
 
     void autoComplete(bool force);
     void autoCompleteColumns(int pos, int len = 0);
     void OnSqlEditUpdateUI(wxStyledTextEvent &event);
     void OnSqlEditCharAdded(wxStyledTextEvent &event);      // autocomplete stuff
+    void OnSqlEditChanged(wxStyledTextEvent &event);        // update title
     void OnSqlEditStartDrag(wxStyledTextEvent& event);      // enable click&remove selection
     wxString keywordsM;     // text used for autocomplete
     void setKeywords();
     void buildMainMenu();
     void buildToolbar();
+    bool gridHasFocus();
 
     // events
     void OnClose(wxCloseEvent& event);
@@ -180,7 +185,9 @@ private:
     void OnMenuUpdateGridInsertRow(wxUpdateUIEvent& event);
     void OnMenuGridDeleteRow(wxCommandEvent &event);
     void OnMenuUpdateGridDeleteRow(wxUpdateUIEvent& event);
-    void OnMenuGridCopy(wxCommandEvent &event);
+    void OnMenuGridImportBlob(wxCommandEvent &event);
+    void OnMenuGridExportBlob(wxCommandEvent &event);
+    void OnMenuUpdateGridCellIsBlob(wxUpdateUIEvent& event);
     void OnMenuGridCopyAsInsert(wxCommandEvent &event);
     void OnMenuGridCopyAsUpdate(wxCommandEvent &event);
     void OnMenuGridSaveAsHtml(wxCommandEvent &event);
