@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: MainFrame.cpp 1636 2008-02-17 22:53:37Z babuskov $
+  $Id: MainFrame.cpp 1692 2008-04-27 17:36:02Z babuskov $
 
 */
 
@@ -59,12 +59,12 @@
 #include "gui/EventWatcherFrame.h"
 #include "gui/ExecuteSql.h"
 #include "gui/MainFrame.h"
+#include "gui/MetadataItemPropertiesFrame.h"
 #include "gui/PreferencesDialog.h"
 #include "gui/ProgressDialog.h"
 #include "gui/RestoreFrame.h"
 #include "gui/ServerRegistrationDialog.h"
 #include "gui/SimpleHtmlFrame.h"
-#include "framemanager.h"
 #include "frtypes.h"
 #include "main.h"
 #include "metadata/metadataitem.h"
@@ -190,7 +190,7 @@ void MainFrame::buildMainMenu()
     viewMenu->AppendSeparator();
     viewMenu->Append(wxID_PREFERENCES, _("P&references..."));
     menuBarM->Append(viewMenu, _("&View"));
-    frameManager().setWindowMenu(viewMenu);
+//    frameManager().setWindowMenu(viewMenu);
 
     wxMenu* serverMenu = new wxMenu();
     serverMenu->Append(Cmds::Menu_RegisterServer, _("&Register server..."));
@@ -290,7 +290,9 @@ void MainFrame::set_properties()
     wxTreeItemIdValue cookie;
     wxTreeItemId firstServer = treeMainM->GetFirstChild(rootNode, cookie);
     if (firstServer.IsOk())
+    {
         treeMainM->SelectItem(firstServer);
+    }
 
     SetIcon(wxArtProvider::GetIcon(ART_FlameRobin, wxART_FRAME_ICON));
 }
@@ -313,7 +315,7 @@ void MainFrame::do_layout()
     searchPanelSizerM->Add(treeMainM, 1, wxEXPAND);
     searchPanelSizerM->Add(searchPanelM, 0, wxEXPAND);
     mainPanelM->SetSizer(searchPanelSizerM);
-
+    
     wxBoxSizer* sizerAll = new wxBoxSizer(wxVERTICAL);
     sizerAll->Add(mainPanelM, 1, wxEXPAND, 0);
     SetAutoLayout(true);
@@ -432,9 +434,9 @@ BEGIN_EVENT_TABLE(MainFrame, wxFrame)
     EVT_MENU(Cmds::Menu_CreateView,       MainFrame::OnMenuCreateView)
 
     EVT_MENU_OPEN(MainFrame::OnMainMenuOpen)
-    EVT_MENU_RANGE(5000, 6000, MainFrame::OnWindowMenuItem)
     EVT_TREE_SEL_CHANGED(DBHTreeControl::ID_tree_ctrl, MainFrame::OnTreeSelectionChanged)
     EVT_TREE_ITEM_ACTIVATED(DBHTreeControl::ID_tree_ctrl, MainFrame::OnTreeItemActivate)
+
     EVT_CLOSE(MainFrame::OnClose)
 END_EVENT_TABLE()
 //-----------------------------------------------------------------------------
@@ -505,6 +507,44 @@ void MainFrame::updateStatusbarText()
 void MainFrame::OnTreeSelectionChanged(wxTreeEvent& WXUNUSED(event))
 {
     updateStatusbarText();
+
+    /* currently disabled until we decide on the new AUI interface for it
+    // switch notebook to show the "Items" page
+    int pg = notebookM->GetPageIndex(labelPanelM);
+    if (pg == wxNOT_FOUND)  // Create it?
+    {
+        return;
+    }
+    notebookM->SetSelection(pg);
+
+    // TODO: listctrl should show what tree shows, but it should also contain
+    //       info about metadata items, provide context menu and double-click
+    //       action for each of them and Observe the items for removal,
+    //       changes and adding new ones.
+    //       i.e. we need a special, separate class for this
+    wxListCtrl *lc = labelPanelM->getListCtrl();
+    lc->SetImageList(treeMainM->GetImageList(), wxIMAGE_LIST_SMALL);
+    lc->ClearAll();
+    wxTreeItemId t = treeMainM->GetSelection();
+    if (!t.IsOk())
+        return;
+    for (wxTreeItemId id = treeMainM->GetLastChild(t); id.IsOk();
+        id = treeMainM->GetPrevSibling(id))
+    {
+        lc->InsertItem(0, treeMainM->GetItemText(id),
+            treeMainM->GetItemImage(id));
+    }
+
+    wxString path;
+    while (t.IsOk())
+    {
+        if (!path.IsEmpty())
+            path = wxT(" > ") + path;
+        path = treeMainM->GetItemText(t) + path;
+        t = treeMainM->GetItemParent(t);
+    }
+    labelPanelM->setLabel(path);
+    */
 }
 //-----------------------------------------------------------------------------
 //! handle double-click on item (or press Enter)
@@ -541,7 +581,7 @@ void MainFrame::OnTreeItemActivate(wxTreeEvent& WXUNUSED(event))
             AddPendingEvent(event);
         }
     }
-    else if (treeActivateAction == selectFromOrExecute 
+    else if (treeActivateAction == selectFromOrExecute
         && (nt == ntTable || nt == ntSysTable || nt == ntView))
     {
         wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED,
@@ -620,7 +660,7 @@ void MainFrame::OnClose(wxCloseEvent& event)
             return;
         }
     }
-    frameManager().setWindowMenu(0);    // tell it not to update menus anymore
+    //frameManager().setWindowMenu(0);    // tell it not to update menus anymore
 
     // the next few lines fix the (threading?) problem on some Linux distributions
     // which leave FlameRobin running if there were connected databases upon exit.
@@ -645,7 +685,7 @@ void MainFrame::OnMenuQuit(wxCommandEvent& WXUNUSED(event))
 //-----------------------------------------------------------------------------
 void MainFrame::OnMenuAbout(wxCommandEvent& WXUNUSED(event))
 {
-	showAboutBox(this);
+    showAboutBox(this);
 }
 //-----------------------------------------------------------------------------
 void MainFrame::OnMenuManual(wxCommandEvent& WXUNUSED(event))
@@ -714,7 +754,7 @@ void MainFrame::OnMenuDatabaseProperties(wxCommandEvent& WXUNUSED(event))
     if (!d)
         return;
 
-    frameManager().showMetadataPropertyFrame(this, d);
+    frameManager().showMetadataPropertyFrame(d);
 }
 //-----------------------------------------------------------------------------
 void MainFrame::OnMenuDatabasePreferences(wxCommandEvent& WXUNUSED(event))
@@ -877,7 +917,7 @@ void MainFrame::OnMenuManageUsers(wxCommandEvent& WXUNUSED(event))
 {
     Server* s = treeMainM->getSelectedServer();
     if (checkValidServer(s))
-        frameManager().showMetadataPropertyFrame(this, s);
+        frameManager().showMetadataPropertyFrame(s);
 }
 //-----------------------------------------------------------------------------
 void MainFrame::OnMenuUnRegisterServer(wxCommandEvent& WXUNUSED(event))
@@ -1390,15 +1430,17 @@ void MainFrame::OnMenuObjectProperties(wxCommandEvent& WXUNUSED(event))
     if (c)
     {
         // Return when we're dealing with a system column
-        if (c->isSystem())
+        if (c->isSystem() || !c->getTable())
             return;
 
-        URI uri = URI(wxT("fr://edit_field?parent_window=") + wxString::Format(wxT("%ld"), (uintptr_t)this)
-            + wxT("&object_address=") + wxString::Format(wxT("%ld"), (uintptr_t)c));
+        URI uri = URI(wxT("fr://edit_field?parent_window=")
+            + wxString::Format(wxT("%ld"), (uintptr_t)this)
+            + wxT("&object_address=") + wxString::Format(wxT("%ld"),
+            (uintptr_t)c));
         getURIProcessor().handleURI(uri);
     }
     else
-        frameManager().showMetadataPropertyFrame(this, m);
+        frameManager().showMetadataPropertyFrame(m);
 }
 //-----------------------------------------------------------------------------
 void MainFrame::OnMenuAlterObject(wxCommandEvent& WXUNUSED(event))
@@ -1423,11 +1465,11 @@ void MainFrame::OnMenuAlterObject(wxCommandEvent& WXUNUSED(event))
 
     wxString sql;
     if (v)
-    	sql = v->getRebuildSql();
+        sql = v->getRebuildSql();
     else if (t)
-    	sql = t->getAlterSql();
+        sql = t->getAlterSql();
     else if (dm)
-    	sql = dm->getAlterSqlTemplate();
+        sql = dm->getAlterSqlTemplate();
     showSql(this, wxString(_("Alter object")), db, sql);
 }
 //-----------------------------------------------------------------------------
