@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: MainFrame.h 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: MainFrame.h 1692 2008-04-27 17:36:02Z babuskov $
 
 */
 
@@ -31,6 +31,7 @@
 #include <wx/wx.h>
 #include <wx/image.h>
 #include <wx/treectrl.h>
+#include <wx/aui/aui.h>
 
 #include <vector>
 
@@ -38,8 +39,10 @@
 #include "metadata/generator.h"
 #include "metadata/metadataitem.h"
 #include "metadata/root.h"
+#include "framemanager.h"
 //-----------------------------------------------------------------------------
 class DBHTreeControl;
+class LabelPanel;
 //-----------------------------------------------------------------------------
 class MainFrame: public BaseFrame
 {
@@ -131,7 +134,8 @@ public:
         ID_button_advanced = 401,
         ID_button_prev,
         ID_button_next,
-        ID_search_box
+        ID_search_box,
+        ID_notebook
     };
     void OnSearchTextChange(wxCommandEvent& event);
     void OnSearchBoxEnter(wxCommandEvent& event);
@@ -144,7 +148,6 @@ public:
         const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE);
 
     bool openUnregisteredDatabase(const wxString& dbpath);
-
 private:
     bool connect();
     void showGeneratorValue(Generator* g);
