@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: MetadataItemPropertiesFrame.cpp 1647 2008-03-11 13:03:05Z mghie $
+  $Id: MetadataItemPropertiesFrame.cpp 1740 2008-09-13 21:27:12Z babuskov $
 
 */
 
@@ -72,21 +72,12 @@ MetadataItemPropertiesFrame::MetadataItemPropertiesFrame(wxWindow* parent,
         MetadataItem *object)
     : BaseFrame(parent, wxID_ANY, wxEmptyString)
 {
-    pageTypeM = ptSummary;
-    objectM = object;
-    htmlReloadRequestedM = false;
-    storageNameM = wxT("unassigned");
-
-    if (!object)
-    {
-        ::wxMessageBox(wxT("MIPF::ctor, Object == 0"), _("Error"), wxOK);
-        return;
-    }
-
-    html_window = new PrintableHtmlWindow(this, HtmlWindowID);
+    // we need to store this right now, since we might lose the object later
+    setStorageName(object);
 
     wxStatusBar *sb = CreateStatusBar();
-    Database* d = objectM->findDatabase();
+
+    Database* d = object->findDatabase();
     if (d)  // server property page doesn't have a database, so don't crash
     {
         wxString s = d->getUsername() + wxT("@") + d->getConnectionString()
@@ -94,24 +85,35 @@ MetadataItemPropertiesFrame::MetadataItemPropertiesFrame(wxWindow* parent,
         sb->SetStatusText(s);
     }
     else
-        sb->SetStatusText(objectM->getPrintableName());
+        sb->SetStatusText(object->getPrintableName());
 
-    // start a transaction for metadata loading and lock the object
-    MetadataLoaderTransaction tr((d) ? d->getMetadataLoader() : 0);
-    SubjectLocker lock(objectM);
-
-    // request initial rendering
-    requestLoadPage(true);
-    objectM->attachObserver(this);
-
-    wxString objName(objectM->getName_());
-    SetTitle(wxString::Format(_("%s: Properties"), objName.c_str()));
-    html_window->SetRelatedFrame(this, objName + wxT(": %s"));
-
-    wxBitmap bmp = getImage32(objectM->getType());
     wxIcon icon;
-    icon.CopyFromBitmap(bmp);
+    if (d && config().get(wxT("linksOpenInTabs"), true))
+    {
+        wxBitmap bmp = getImage32(d->getType());
+        icon.CopyFromBitmap(bmp);
+        databaseNameM = d->getPrintableName();
+    }
+    else  // when linksOpenInTabs, only the server node
+    {
+        wxBitmap bmp = getImage32(object->getType());
+        icon.CopyFromBitmap(bmp);
+    }
     SetIcon(icon);
+
+    notebookM = new wxAuiNotebook(this, ID_notebook, wxDefaultPosition,
+        wxDefaultSize, wxAUI_NB_DEFAULT_STYLE | wxAUI_NB_WINDOWLIST_BUTTON
+        | wxAUI_NB_TAB_EXTERNAL_MOVE | wxNO_BORDER );
+
+    auiManagerM.SetManagedWindow(this);
+    auiManagerM.AddPane(notebookM,
+        wxAuiPaneInfo().CenterPane().PaneBorder(false));
+    auiManagerM.Update();
+
+    Connect(wxEVT_COMMAND_AUINOTEBOOK_PAGE_CLOSE, wxAuiNotebookEventHandler(
+        MetadataItemPropertiesFrame::OnNotebookPageClose), NULL, this);
+    Connect(wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGED, wxAuiNotebookEventHandler(
+        MetadataItemPropertiesFrame::OnNotebookPageChanged), NULL, this);
 }
 //-----------------------------------------------------------------------------
 const wxRect MetadataItemPropertiesFrame::getDefaultRect() const
@@ -124,40 +126,122 @@ const wxString MetadataItemPropertiesFrame::getName() const
     return wxT("MIPFrame");
 }
 //-----------------------------------------------------------------------------
-MetadataItem *MetadataItemPropertiesFrame::getObservedObject() const
+const wxString MetadataItemPropertiesFrame::getStorageName() const
+{
+    return storageNameM;
+}
+//-----------------------------------------------------------------------------
+void MetadataItemPropertiesFrame::removePanel(wxWindow* panel)
+{
+    int pg = notebookM->GetPageIndex(panel);
+    if (pg == wxNOT_FOUND)
+        return;
+
+    notebookM->DeletePage(pg);
+    if (notebookM->GetPageCount() < 1)
+        Close();
+}
+//-----------------------------------------------------------------------------
+void MetadataItemPropertiesFrame::setStorageName(MetadataItem *object)
+{
+    StorageGranularity g;
+    if (!config().getValue(wxT("MetadataFrameStorageGranularity"), g))
+        g = sgFrame;
+    if (config().get(wxT("linksOpenInTabs"), true))
+        g = sgFrame;
+
+    switch (g)
+    {
+        case sgFrame:
+            storageNameM = getName();
+            break;
+        case sgObjectType:
+            storageNameM = getName() + Config::pathSeparator
+                + object->getTypeName();
+            break;
+        case sgObject:
+            storageNameM = getName() + Config::pathSeparator
+                + object->getItemPath();
+            break;
+        default:
+            storageNameM = wxT("");
+            break;
+    }
+}
+//-----------------------------------------------------------------------------
+void MetadataItemPropertiesFrame::setTabTitle(wxWindow *panel,
+    const wxString& title)
+{
+    int pg = notebookM->GetPageIndex(panel);
+    if (pg == wxNOT_FOUND)
+        return;
+    notebookM->SetPageText(pg, title);
+}
+//-----------------------------------------------------------------------------
+void MetadataItemPropertiesFrame::showPanel(wxWindow* panel,
+    const wxString& title)
+{
+    int pg = notebookM->GetPageIndex(panel);
+    if (pg == wxNOT_FOUND)
+        notebookM->AddPage(panel, title, true);
+    else
+        notebookM->SetSelection(pg);
+
+    Show();
+    Raise();
+}
+//-----------------------------------------------------------------------------
+MetadataItemPropertiesPanel::MetadataItemPropertiesPanel(
+    MetadataItemPropertiesFrame* parent, MetadataItem *object)
+    :wxPanel(parent, wxID_ANY)
+{
+    pageTypeM = ptSummary;
+    objectM = object;
+    htmlReloadRequestedM = false;
+
+    if (!object)
+    {
+        ::wxMessageBox(wxT("MIPF::ctor, Object == 0"), _("Error"), wxOK);
+        return;
+    }
+
+    html_window = new PrintableHtmlWindow(this, HtmlWindowID);
+    parent->SetTitle(object->getName_());
+
+    wxBoxSizer* bSizer2 = new wxBoxSizer( wxVERTICAL );
+    bSizer2->Add( html_window, 1, wxEXPAND, 0 );
+    SetSizer( bSizer2 );
+    Layout();
+
+    Database* d = objectM->findDatabase();
+    // start a transaction for metadata loading and lock the object
+    MetadataLoaderTransaction tr((d) ? d->getMetadataLoader() : 0);
+    SubjectLocker lock(objectM);
+
+    // request initial rendering
+    requestLoadPage(true);
+    objectM->attachObserver(this);
+}
+//-----------------------------------------------------------------------------
+MetadataItemPropertiesPanel::~MetadataItemPropertiesPanel()
+{
+    frameManager().removeFrame(this);
+}
+//-----------------------------------------------------------------------------
+void MetadataItemPropertiesPanel::showIt()
+{
+    MetadataItemPropertiesFrame *f = getParentFrame();
+    if (f)
+        f->showPanel(this, objectM->getName_());
+}
+//-----------------------------------------------------------------------------
+MetadataItem *MetadataItemPropertiesPanel::getObservedObject() const
 {
     return objectM;
 }
 //-----------------------------------------------------------------------------
-const wxString MetadataItemPropertiesFrame::getStorageName() const
-{
-    if (storageNameM == wxT("unassigned"))
-    {
-        StorageGranularity g;
-        if (!config().getValue(wxT("MetadataFrameStorageGranularity"), g))
-            g = sgFrame;
-
-        switch (g)
-        {
-            case sgFrame:
-                storageNameM = getName();
-                break;
-            case sgObjectType:
-                storageNameM = getName() + Config::pathSeparator + objectM->getTypeName();
-                break;
-            case sgObject:
-                storageNameM = getName() + Config::pathSeparator + objectM->getItemPath();
-                break;
-            default:
-                storageNameM = wxT("");
-                break;
-        }
-    }
-    return storageNameM;
-}
-//-----------------------------------------------------------------------------
 //! defer (possibly expensive) creation and display of html page to idle time
-void MetadataItemPropertiesFrame::requestLoadPage(bool showLoadingPage)
+void MetadataItemPropertiesPanel::requestLoadPage(bool showLoadingPage)
 {
     if (!htmlReloadRequestedM)
     {
@@ -169,13 +253,13 @@ void MetadataItemPropertiesFrame::requestLoadPage(bool showLoadingPage)
         }
 
         Connect(wxID_ANY, wxEVT_IDLE,
-            wxIdleEventHandler(MetadataItemPropertiesFrame::OnIdle));
+            wxIdleEventHandler(MetadataItemPropertiesPanel::OnIdle));
         htmlReloadRequestedM = true;
     }
 }
 //-----------------------------------------------------------------------------
 //! determine the path, load and display html page
-void MetadataItemPropertiesFrame::loadPage()
+void MetadataItemPropertiesPanel::loadPage()
 {
     wxString htmlpage = config().getHtmlTemplatesPath();
     switch (pageTypeM)
@@ -217,7 +301,7 @@ void MetadataItemPropertiesFrame::loadPage()
 //
 //! command is in format:   {%action:data%}
 //! data field can be empty
-void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *object, wxString& htmlpage)
+void MetadataItemPropertiesPanel::processCommand(wxString cmd, MetadataItem *object, wxString& htmlpage)
 {
     wxString::size_type pos = cmd.find(':');
     wxString suffix;
@@ -261,6 +345,8 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
         pages.push_back(wxT("Summary"));
         if (object->getType() == ntRole)        // special case, roles
             pages.push_back(wxT("Privileges")); // don't have dependencies
+        if (object->getType() == ntDatabase)
+            pages.push_back(wxT("Triggers"));   // FB 2.1
         switch (object->getType())
         {
             case ntSysTable:
@@ -317,7 +403,7 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
         if (!s)
             return;
 
-        ProgressDialog pd(this, _("Connecting to Server..."), 1);
+        ProgressDialog pd(GetParent(), _("Connecting to Server..."), 1);
         UserList* usr = s->getUsers(&pd);
         if (!usr || !usr->size())
         {
@@ -404,17 +490,24 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
         }
     }
 
-    else if (cmd == wxT("triggers")) // table triggers,  triggers:after or triggers:befor  <- not a typo
+    // table triggers,  triggers:after or triggers:befor  <- not a typo
+    else if (cmd == wxT("triggers"))
     {
         Relation* r = dynamic_cast<Relation*>(object);
-        if (!r)
+        Database* d = dynamic_cast<Database*>(object);
+        if (!r && !d)
             return;
         std::vector<Trigger*> tmp;
-        if (suffix.substr(0, 5) == wxT("after"))
-            r->getTriggers(tmp, Trigger::afterTrigger);
-        else
-            r->getTriggers(tmp, Trigger::beforeTrigger);
-        suffix.erase(0, 5);
+        if (r)
+        {
+            if (suffix.substr(0, 5) == wxT("after"))
+                r->getTriggers(tmp, Trigger::afterTrigger);
+            else
+                r->getTriggers(tmp, Trigger::beforeTrigger);
+            suffix.erase(0, 5);
+        }
+        else // d
+            d->getDatabaseTriggers(tmp);
         for (std::vector<Trigger*>::iterator it = tmp.begin(); it != tmp.end(); ++it)
             processHtmlCode(htmlpage, suffix, *it);
     }
@@ -696,13 +789,16 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
     {
         Trigger* t = dynamic_cast<Trigger*>(object);
         wxString object, type;
-        bool active;
+        bool active, isDBtrigger;
         int position;
         if (!t)
             return;
-        t->getTriggerInfo(object, active, position, type);
+        t->getTriggerInfo(object, active, position, type, isDBtrigger);
         wxString s(active ? wxT("Active ") : wxT("Inactive "));
-        s << type << wxT(" trigger for ") << object << wxT(" at position ") << position;
+        s << type << wxT(" trigger ");
+        if (!isDBtrigger)
+            s << wxT("for ") << object;
+        s << wxT(" at position ") << position;
         htmlpage += escapeHtmlChars(s, false);
     }
 
@@ -768,7 +864,7 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
 
     else if (cmd == wxT("object_ddl"))
     {
-        ProgressDialog pd(this, _("Extracting DDL Definitions"), 2);
+        ProgressDialog pd(GetParent(), _("Extracting DDL Definitions"), 2);
 
         CreateDDLVisitor cdv(&pd);
         object->acceptVisitor(&cdv);
@@ -932,7 +1028,7 @@ void MetadataItemPropertiesFrame::processCommand(wxString cmd, MetadataItem *obj
 }
 //-----------------------------------------------------------------------------
 //! processes html template code given in the htmlsource wxString
-void MetadataItemPropertiesFrame::processHtmlCode(wxString& htmlpage, wxString htmlsource, MetadataItem *object)
+void MetadataItemPropertiesPanel::processHtmlCode(wxString& htmlpage, wxString htmlsource, MetadataItem *object)
 {
     if (object == 0)
         object = objectM;
@@ -981,7 +1077,7 @@ void MetadataItemPropertiesFrame::processHtmlCode(wxString& htmlpage, wxString h
 }
 //-----------------------------------------------------------------------------
 //! processes the given html template file
-void MetadataItemPropertiesFrame::processHtmlFile(wxString fileName)
+void MetadataItemPropertiesPanel::processHtmlFile(wxString fileName)
 {
     wxString htmlpage;
     processHtmlCode(htmlpage, loadEntireFile(fileName));
@@ -990,21 +1086,56 @@ void MetadataItemPropertiesFrame::processHtmlFile(wxString fileName)
     html_window->GetViewStart(&x, &y);         // save scroll position
     html_window->setPageSource(htmlpage);
     html_window->Scroll(x, y);                 // restore scroll position
+
+    // set title
+    if (getParentFrame())
+    {
+        getParentFrame()->setTabTitle(this, objectM->getName_() + wxT(": ")
+            + html_window->GetOpenedPageTitle());
+    }
 }
 //-----------------------------------------------------------------------------
 //! closes window if observed object gets removed (disconnecting, dropping, etc)
-void MetadataItemPropertiesFrame::removeSubject(Subject* subject)
+void MetadataItemPropertiesPanel::removeSubject(Subject* subject)
 {
     Observer::removeSubject(subject);
     // main observed object is getting destroyed
     if (subject == objectM)
     {
         objectM = 0;
-        Close();
+        MetadataItemPropertiesFrame *f = getParentFrame();
+        if (f)
+            f->removePanel(this);
     }
 }
 //-----------------------------------------------------------------------------
-void MetadataItemPropertiesFrame::setPage(const wxString& type)
+MetadataItemPropertiesFrame* MetadataItemPropertiesPanel::getParentFrame()
+{
+    for (wxWindow *w = GetParent(); w; w = w->GetParent())
+    {
+        MetadataItemPropertiesFrame *f = dynamic_cast<
+            MetadataItemPropertiesFrame *>(w);
+        if (f)
+            return f;
+    }
+    return 0;
+}
+//-----------------------------------------------------------------------------
+/*
+MetadataItemPropertiesPanel *MetadataItemPropertiesFrame::getItemPanel(
+    MetadataItem *item)
+{
+    for (size_t c = 0; c < notebookM->GetPageCount(); c++)
+    {
+        MetadataItemPropertiesPanel *p = dynamic_cast<
+            MetadataItemPropertiesPanel *>(notebookM->GetPage(c));
+        if (p && p->getObservedObject() == item)
+            return p;
+    }
+    return 0;
+}*/
+//-----------------------------------------------------------------------------
+void MetadataItemPropertiesPanel::setPage(const wxString& type)
 {
     if (type == wxT("constraints"))
         pageTypeM = ptConstraints;
@@ -1025,13 +1156,22 @@ void MetadataItemPropertiesFrame::setPage(const wxString& type)
 }
 //-----------------------------------------------------------------------------
 //! recreate html page if something changes
-void MetadataItemPropertiesFrame::update()
+void MetadataItemPropertiesPanel::update()
 {
     Database *db = dynamic_cast<Database *>(objectM);
     if (db && !db->isConnected())
     {
         objectM = 0;
-        Close();
+        MetadataItemPropertiesFrame* f = getParentFrame();
+        if (f)
+            f->Close();
+
+            // MB: This code used to use:
+            //f->removePanel(this);
+            // which would allow us to mix property pages from different
+            // databases in the same Frame, but there are some mysterious
+            // reasons why it causes heap corruption with MSVC
+
         return;
     }
 
@@ -1074,13 +1214,50 @@ void MetadataItemPropertiesFrame::update()
 }
 //-----------------------------------------------------------------------------
 #if wxCHECK_VERSION(2, 8, 0)
-BEGIN_EVENT_TABLE(MetadataItemPropertiesFrame, BaseFrame)
-    EVT_HTML_CELL_HOVER(MetadataItemPropertiesFrame::HtmlWindowID,
-        MetadataItemPropertiesFrame::OnHtmlCellHover)
+BEGIN_EVENT_TABLE(MetadataItemPropertiesPanel, wxPanel)
+    EVT_HTML_CELL_HOVER(MetadataItemPropertiesPanel::HtmlWindowID,
+        MetadataItemPropertiesPanel::OnHtmlCellHover)
 END_EVENT_TABLE()
 #endif
 //-----------------------------------------------------------------------------
-void MetadataItemPropertiesFrame::OnIdle(wxIdleEvent& WXUNUSED(event))
+BEGIN_EVENT_TABLE(MetadataItemPropertiesFrame, BaseFrame)
+    EVT_CLOSE(MetadataItemPropertiesFrame::OnClose)
+END_EVENT_TABLE()
+//-----------------------------------------------------------------------------
+// when last tab is closed, close the frame
+void MetadataItemPropertiesFrame::OnNotebookPageClose(wxAuiNotebookEvent&
+    WXUNUSED(event))
+{
+    // seems that page count returns pages before event not after
+    // probably because event can be Vetoed
+    if (notebookM->GetPageCount() < 2)
+        Close();
+}
+//-----------------------------------------------------------------------------
+// when last tab is closed, close the frame
+void MetadataItemPropertiesFrame::OnNotebookPageChanged(wxAuiNotebookEvent&
+    event)
+{
+    int sel = event.GetSelection();
+    if (sel == wxNOT_FOUND)
+        return;
+    if (databaseNameM.IsEmpty())
+        SetTitle(notebookM->GetPageText(sel));
+    else
+        SetTitle(databaseNameM + wxT(" - ") + notebookM->GetPageText(sel));
+}
+//-----------------------------------------------------------------------------
+void MetadataItemPropertiesFrame::OnClose(wxCloseEvent& event)
+{
+    Disconnect(wxEVT_COMMAND_AUINOTEBOOK_PAGE_CLOSE, wxAuiNotebookEventHandler(
+        MetadataItemPropertiesFrame::OnNotebookPageClose), NULL, this);
+    Disconnect(wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGED,
+        wxAuiNotebookEventHandler(
+        MetadataItemPropertiesFrame::OnNotebookPageChanged), NULL, this);
+    BaseFrame::OnClose(event);
+}
+//-----------------------------------------------------------------------------
+void MetadataItemPropertiesPanel::OnIdle(wxIdleEvent& WXUNUSED(event))
 {
     Disconnect(wxID_ANY, wxEVT_IDLE);
     wxBusyCursor bc;
@@ -1089,7 +1266,7 @@ void MetadataItemPropertiesFrame::OnIdle(wxIdleEvent& WXUNUSED(event))
 }
 //-----------------------------------------------------------------------------
 #if wxCHECK_VERSION(2, 8, 0)
-void MetadataItemPropertiesFrame::OnHtmlCellHover(wxHtmlCellEvent& event)
+void MetadataItemPropertiesPanel::OnHtmlCellHover(wxHtmlCellEvent& event)
 {
     wxHtmlCell *c = event.GetCell();
     if (!c)
@@ -1133,27 +1310,25 @@ bool PageHandler::handleURI(URI& uri)
     if (uri.action != wxT("page"))
         return false;
 
-    wxString ms = uri.getParam(wxT("parent_window"));     // window
-    unsigned long mo;
-    if (!ms.ToULong(&mo))
+    MetadataItemPropertiesPanel* mpp = dynamic_cast<
+        MetadataItemPropertiesPanel*>(getWindow(uri));
+    if (!mpp)
         return true;
-    MetadataItemPropertiesFrame* mpf = (MetadataItemPropertiesFrame*)mo;
+
     if (uri.getParam(wxT("target")) == wxT("new"))
     {
-        wxWindow* mainFrame = mpf->GetParent();
-        if (mainFrame)
-        {
-            mpf = frameManager().showMetadataPropertyFrame(mainFrame,
-                                    // !delayed, force_new
-                mpf->getObservedObject(), false, true);
-        }
+        mpp = frameManager().showMetadataPropertyFrame(
+            mpp->getObservedObject(), false, true); // !delayed, force_new
+    }
+    else if (uri.getParam(wxT("target")) == wxT("new_tab"))
+    {
+        mpp = frameManager().showMetadataPropertyFrame(
+            mpp->getObservedObject(), false, false, true,
+            mpp->getParentFrame()); // true = new_tab
     }
 
-    if (mpf)
-    {
-        mpf->setPage(uri.getParam(wxT("type")));
-        frameManager().rebuildMenu();
-    }
+    mpp->setPage(uri.getParam(wxT("type")));
+    //frameManager().rebuildMenu();
     return true;
 }
 //-----------------------------------------------------------------------------
@@ -1172,25 +1347,28 @@ bool PropertiesHandler::handleURI(URI& uri)
     if (uri.action != wxT("properties"))
         return false;
 
-    MetadataItemPropertiesFrame* parent = dynamic_cast<MetadataItemPropertiesFrame*>(getWindow(uri));
+    MetadataItemPropertiesPanel* parent = dynamic_cast<
+        MetadataItemPropertiesPanel*>(getWindow(uri));
     if (!parent)
         return true;
     Database* d = parent->getObservedObject()->findDatabase();
     if (!d)
         return true;
     NodeType n = getTypeByName(uri.getParam(wxT("object_type")));
-    MetadataItem* object = d->findByNameAndType(n, uri.getParam(wxT("object_name")));
+    MetadataItem* object = d->findByNameAndType(n,
+        uri.getParam(wxT("object_name")));
     if (!object)
     {
-        ::wxMessageBox(_("Cannot find destination object\nThis should never happen."), _("Error"), wxICON_ERROR);
+        ::wxMessageBox(
+            _("Cannot find destination object\nThis should never happen."),
+            _("Error"), wxICON_ERROR);
         return true;
     }
 
-    // check if window with properties of that object is already open and show it
-    wxWindow* mainFrame = parent->GetParent();
-    if (mainFrame)
-        frameManager().showMetadataPropertyFrame(mainFrame, object, false,
-            uri.getParam(wxT("target")) == wxT("new"));
+    frameManager().showMetadataPropertyFrame(object, false,
+        uri.getParam(wxT("target")) == wxT("new"),
+        uri.getParam(wxT("target")) == wxT("new_tab"),
+        parent->getParentFrame());
     return true;
 }
 //-----------------------------------------------------------------------------
