@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: MetadataItemPropertiesFrame.h 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: MetadataItemPropertiesFrame.h 1740 2008-09-13 21:27:12Z babuskov $
 
 */
 
@@ -30,24 +30,27 @@
 //-----------------------------------------------------------------------------
 #include <wx/wx.h>
 #include <wx/wxhtml.h>
+#include <wx/aui/aui.h>
 
 // wx 2.6 doesn't support wxHtmlCellEvent
 #if wxCHECK_VERSION(2, 8, 0)
 #include <wx/html/htmlwin.h>
 #endif
 
+#include "framemanager.h"
 #include "core/Observer.h"
 #include "gui/BaseFrame.h"
 #include "gui/controls/PrintableHtmlWindow.h"
 #include "metadata/metadataitem.h"
+class MetadataItemPropertiesFrame;
 //-----------------------------------------------------------------------------
-class MetadataItemPropertiesFrame: public BaseFrame, public Observer
+class MetadataItemPropertiesPanel: public wxPanel, public Observer
 {
 private:
     enum { ptSummary, ptConstraints, ptDependencies, ptTriggers,
         ptTableIndices, ptDDL, ptPrivileges } pageTypeM;
 
-	enum { HtmlWindowID = 42 };
+    enum { HtmlWindowID = 42 };
 
     MetadataItem *objectM;
     bool htmlReloadRequestedM;
@@ -61,22 +64,19 @@ private:
     void processHtmlCode(wxString& htmlpage, wxString htmlsource,
         MetadataItem* object = 0);
 
-    // used to remember the value among calls to getStorageName(),
-    // needed because it's not possible to access objectM
-    // (see getStorageName()) after detaching from it.
-    mutable wxString storageNameM;
 protected:
-    virtual const wxString getName() const;
-    virtual const wxString getStorageName() const;
-    virtual const wxRect getDefaultRect() const;
     virtual void removeSubject(Subject* subject);
     virtual void update();
 public:
-    MetadataItemPropertiesFrame(wxWindow* parent, MetadataItem *object);
+    MetadataItemPropertiesPanel(MetadataItemPropertiesFrame* parent,
+        MetadataItem *object);
+    virtual ~MetadataItemPropertiesPanel();
 
     MetadataItem* getObservedObject() const;
     void processHtmlFile(wxString fileName);
     void setPage(const wxString& type);
+    void showIt();
+    MetadataItemPropertiesFrame *getParentFrame();
 private:
     // event handling
     void OnIdle(wxIdleEvent& event);
@@ -87,4 +87,43 @@ private:
 #endif
 };
 //-----------------------------------------------------------------------------
+class MetadataItemPropertiesFrame: public BaseFrame
+{
+private:
+    wxString databaseNameM;
+
+    // used to remember the value among calls to getStorageName(),
+    // needed because it's not possible to access objectM
+    // (see getStorageName()) after detaching from it.
+    mutable wxString storageNameM;
+    void setStorageName(MetadataItem *object);
+protected:
+    virtual const wxString getName() const;
+    virtual const wxString getStorageName() const;
+    virtual const wxRect getDefaultRect() const;
+public:
+    MetadataItemPropertiesFrame(wxWindow* parent, MetadataItem *object);
+    virtual ~MetadataItemPropertiesFrame()
+    {
+        auiManagerM.UnInit();
+    }
+
+    enum { ID_notebook = 43 };
+
+    void showPanel(wxWindow *panel, const wxString& title);
+    void removePanel(wxWindow *panel);
+    void setTabTitle(wxWindow *panel, const wxString& title);
+
+    //MetadataItemPropertiesPanel *getItemPanel(MetadataItem *item);
+
+    void OnClose(wxCloseEvent& event);
+    void OnNotebookPageClose(wxAuiNotebookEvent& event);
+    void OnNotebookPageChanged(wxAuiNotebookEvent& event);
+private:
+    wxAuiManager auiManagerM;
+    wxAuiNotebook* notebookM;
+
+    DECLARE_EVENT_TABLE()
+};
+//-----------------------------------------------------------------------------
 #endif // FR_METADATAITEMPROPERTIESFRAME_H
