@@ -21,7 +21,7 @@ TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: PreferencesDialogSettings.cpp 1249 2006-11-17 16:06:49Z mbabuskov $
+  $Id: PreferencesDialogSettings.cpp 1287 2007-05-10 16:25:39Z mghie $
 
 */
 
@@ -813,7 +813,12 @@ void PrefDlgChooserSetting::chooseFile()
     wxFileName::SplitPath(textctrlM->GetValue(), &path, 0, 0);
 
     wxString filename = ::wxFileSelector(_("Select File"), path,
-        wxEmptyString, wxEmptyString, _("All files (*.*)|*.*"), wxSAVE,
+        wxEmptyString, wxEmptyString, _("All files (*.*)|*.*"),
+#if wxCHECK_VERSION(2, 8, 0)
+        wxFD_SAVE,
+#else
+        wxSAVE,
+#endif
         ::wxGetTopLevelParent(textctrlM));
     if (!filename.IsEmpty())
         textctrlM->SetValue(filename);
@@ -864,7 +869,7 @@ wxStaticText* PrefDlgChooserSetting::getLabel()
 //-----------------------------------------------------------------------------
 bool PrefDlgChooserSetting::hasControls() const
 {
-    return (captionM) || (textctrlM) ||(browsebtnM);
+    return (captionBeforeM) || (textctrlM) ||(browsebtnM);
 }
 //-----------------------------------------------------------------------------
 bool PrefDlgChooserSetting::loadFromConfig(Config& config)
