@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: RestoreFrame.cpp 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: RestoreFrame.cpp 1654 2008-03-19 14:38:07Z mghie $
 
 */
 
@@ -117,7 +117,7 @@ void* RestoreThread::Entry()
             if (TestDestroy())
             {
                 now = wxDateTime::Now();
-                msg.Printf(_("Database restore cancelled %s"),
+                msg.Printf(_("Database restore canceled %s"),
                     now.FormatTime().c_str());
                 logImportant(msg);
                 break;
@@ -139,7 +139,7 @@ void* RestoreThread::Entry()
     catch (IBPP::Exception& e)
     {
         now = wxDateTime::Now();
-        msg.Printf(_("Database restore cancelled %s due to IBPP exception:\n\n"),
+        msg.Printf(_("Database restore canceled %s due to IBPP exception:\n\n"),
             now.FormatTime().c_str());
         msg += std2wx(e.ErrorMessage());
         logError(msg);
@@ -147,7 +147,7 @@ void* RestoreThread::Entry()
     catch (...)
     {
         now = wxDateTime::Now();
-        msg.Printf(_("Database restore cancelled %s due to exception"),
+        msg.Printf(_("Database restore canceled %s due to exception"),
             now.FormatTime().c_str());
         logError(msg);
     }
