@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: DataGridRows.cpp 1635 2008-02-10 17:35:55Z mghie $
+  $Id: DataGridRows.cpp 1744 2008-09-15 11:14:06Z babuskov $
 
 */
 
@@ -39,6 +39,7 @@
 #endif
 
 #include <wx/datetime.h>
+#include <wx/ffile.h>
 #include <wx/textbuf.h>
 
 #include <algorithm>
@@ -50,6 +51,7 @@
 #include "config/Config.h"
 #include "core/FRError.h"
 #include "core/Observer.h"
+#include "core/ProgressIndicator.h"
 #include "core/StringUtils.h"
 #include "frtypes.h"
 #include "gui/controls/DataGridRowBuffer.h"
@@ -63,6 +65,7 @@ private:
     int precisionForDoubleM;
     wxString dateFormatM;
     wxString timeFormatM;
+    wxString timestampFormatM;
 protected:
     virtual void loadFromConfig();
 public:
@@ -73,10 +76,14 @@ public:
     wxString formatDouble(double value);
     wxString formatDate(int year, int month, int day);
     wxString formatTime(int hour, int minute, int second, int milliSecond);
+    wxString formatTimestamp(int year, int month, int day,
+        int hour, int minute, int second, int milliSecond);
     bool parseDate(wxString::iterator& start, wxString::iterator end,
         bool consumeAll, int& year, int& month, int& day);
     bool parseTime(wxString::iterator& start, wxString::iterator end,
         int& hr, int& mn, int& sc, int& ml);
+    bool parseTimestamp(wxString::iterator& start, wxString::iterator end,
+        int& year, int& month, int& day, int& hr, int& mn, int& sc, int& ml);
 };
 //-----------------------------------------------------------------------------
 GridCellFormats::GridCellFormats()
@@ -98,6 +105,8 @@ void GridCellFormats::loadFromConfig()
 
     dateFormatM = config().get(wxT("DateFormat"), wxString(wxT("D.M.Y")));
     timeFormatM = config().get(wxT("TimeFormat"), wxString(wxT("H:M:S.T")));
+    timestampFormatM = config().get(wxT("TimestampFormat"),
+        wxString(wxT("D.N.Y, H:M:S.T")));
 }
 //-----------------------------------------------------------------------------
 wxString GridCellFormats::formatDouble(double value)
@@ -306,6 +315,146 @@ bool GridCellFormats::parseTime(wxString::iterator& start,
     return true;
 }
 //-----------------------------------------------------------------------------
+wxString GridCellFormats::formatTimestamp(int year, int month, int day,
+    int hour, int minute, int second, int milliSecond)
+{
+    ensureCacheValid();
+
+    wxString result;
+    for (wxString::iterator c = timestampFormatM.begin();
+        c != timestampFormatM.end(); c++)
+    {
+        switch ((wxChar)*c)
+        {
+            case 'd':
+                result += wxString::Format(wxT("%d"), day);
+                break;
+            case 'D':
+                result += wxString::Format(wxT("%02d"), day);
+                break;
+            case 'n':
+                result += wxString::Format(wxT("%d"), month);
+                break;
+            case 'N':
+                result += wxString::Format(wxT("%02d"), month);
+                break;
+            case 'y':
+                result += wxString::Format(wxT("%02d"), year % 100);
+                break;
+            case 'Y':
+                result += wxString::Format(wxT("%04d"), year);
+                break;
+            case 'h':
+                result += wxString::Format(wxT("%d"), hour);
+                break;
+            case 'H':
+                result += wxString::Format(wxT("%02d"), hour);
+                break;
+            case 'm':
+                result += wxString::Format(wxT("%d"), minute);
+                break;
+            case 'M':
+                result += wxString::Format(wxT("%02d"), minute);
+                break;
+            case 's':
+                result += wxString::Format(wxT("%d"), second);
+                break;
+            case 'S':
+                result += wxString::Format(wxT("%02d"), second);
+                break;
+            case 'T':
+                result += wxString::Format(wxT("%03d"), milliSecond);
+                break;
+            default:
+                result += *c;
+                break;
+        }
+    }
+    return result;
+}
+//-----------------------------------------------------------------------------
+bool GridCellFormats::parseTimestamp(wxString::iterator& start,
+    wxString::iterator end, int& year, int& month, int& day,
+    int& hr, int& mn, int& sc, int& ml)
+{
+    ensureCacheValid();
+
+    for (wxString::iterator c = timestampFormatM.begin();
+        c != timestampFormatM.end() && start != end; ++c)
+    {
+        switch ((wxChar)*c)
+        {
+            case 'd':
+            case 'D':
+                if (*start < wxChar('0') || *start > wxChar('9'))
+                    return true;
+                if (!(getNumber(start, day) && day >= 1 && day <= 31))
+                    return false;
+                break;
+            case 'n':
+            case 'N':
+                if (*start < wxChar('0') || *start > wxChar('9'))
+                    return true;
+                if (!(getNumber(start, month) && month >= 1 && month <= 12))
+                    return false;
+                break;
+            case 'y':
+                if (*start < wxChar('0') || *start > wxChar('9'))
+                    return true;
+                if (!getNumber(start, year))
+                    return false;
+                // see http://www.firebirdsql.org/doc/contrib/FirebirdDateLiterals.html
+                if (year < 100)
+                {
+                    int thisYear = wxDateTime::Now().GetYear();
+                    int cy = thisYear / 100;
+                    int yearBefore = 100 * cy + year;
+                    int yearAfter = yearBefore;
+                    if (yearBefore > thisYear)
+                        yearBefore -= 100;
+                    else
+                        yearAfter += 100;
+                    if (thisYear - yearBefore <= yearAfter - thisYear)
+                        year = yearBefore;
+                    else
+                        year = yearAfter;
+                }
+                break;
+            case 'Y':
+                if (*start < wxChar('0') || *start > wxChar('9'))
+                    return true;
+                if (!getNumber(start, year))
+                    return false;
+                break;
+            case 'h':
+            case 'H':
+                if (!(getNumber(start, hr) && hr >= 0 && hr <= 23))
+                    return false;
+                break;
+            case 'm':
+            case 'M':
+                if (!(getNumber(start, mn) && mn >= 0 && mn <= 59))
+                    return false;
+                break;
+            case 's':
+            case 'S':
+                if (!(getNumber(start, sc) && sc >= 0 && sc <= 59))
+                    return false;
+                break;
+            case 'T':
+                if (!(getNumber(start, ml) && ml >= 0 && ml <= 999))
+                    return false;
+                break;
+            default:        // other characters must match
+                if (*c != *start)
+                    return false;
+                ++start;
+                break;
+        }
+    }
+    return true;
+}
+//-----------------------------------------------------------------------------
 // ResultsetColumnDef class
 ResultsetColumnDef::ResultsetColumnDef(const wxString& name, bool readonly,
     bool nullable)
@@ -809,15 +958,8 @@ wxString TimestampColumnDef::getAsString(DataGridRowBuffer* buffer)
     date.GetDate(year, month, day);
     time.GetTime(hour, minute, second, tenththousands);
 
-    wxString dateStr = GridCellFormats::get().formatDate(year, month, day);
-    wxString timeStr = GridCellFormats::get().formatTime(hour, minute, second,
-        tenththousands / 10);
-    if (timeStr.empty())
-        return dateStr;
-    else if (dateStr.empty())
-        return timeStr;
-    else
-        return dateStr + wxT(", ") + timeStr;
+    return GridCellFormats::get().formatTimestamp(year, month, day,
+        hour, minute, second, tenththousands / 10);
 }
 //-----------------------------------------------------------------------------
 wxString TimestampColumnDef::getAsFirebirdString(DataGridRowBuffer* buffer)
@@ -836,10 +978,8 @@ wxString TimestampColumnDef::getAsFirebirdString(DataGridRowBuffer* buffer)
     date.GetDate(year, month, day);
     time.GetTime(hour, minute, second, tenththousands);
 
-    wxString dateStr = wxString::Format(wxT("%d-%d-%d"), year, month, day);
-    wxString timeStr = wxString::Format(wxT("%d:%d:%d.%d"), hour, minute,
-        second, tenththousands / 10);
-    return dateStr + wxT(", ") + timeStr;
+    return wxString::Format(wxT("%d-%d-%d, %d:%d:%d.%d"), year, month, day,
+        hour, minute, second, tenththousands / 10);
 }
 //-----------------------------------------------------------------------------
 void TimestampColumnDef::setFromString(DataGridRowBuffer* buffer,
@@ -866,20 +1006,14 @@ void TimestampColumnDef::setFromString(DataGridRowBuffer* buffer,
         int y = its.Year();  // defaults
         int m = its.Month();
         int d = its.Day();
-        wxString::iterator it = temp.begin();
-        // do not consume all chars, leave them for (optional) time value
-        if (!GridCellFormats::get().parseDate(it, temp.end(), false, y, m, d))
-            throw FRError(_("Cannot parse date"));
-        its.SetDate(y, m, d);
-
-        // skip spaces and commas ", "
-        while ((wxChar)*it == wxChar(',') || (wxChar)*it == wxChar(' '))
-            it++;
-
-        // get time (if available)
         int hr = 0, mn = 0, sc = 0, ms = 0;
-        if (!GridCellFormats::get().parseTime(it, temp.end(), hr, mn, sc, ms))
-            throw FRError(_("Cannot parse time"));
+        wxString::iterator it = temp.begin();
+        if (!GridCellFormats::get().parseTimestamp(it, temp.end(),
+            y, m, d, hr, mn, sc, ms))
+        {
+            throw FRError(_("Cannot parse timestamp"));
+        }
+        its.SetDate(y, m, d);
         its.SetTime(hr, mn, sc, 10 * ms);
     }
 
@@ -1042,9 +1176,11 @@ class BlobColumnDef : public ResultsetColumnDef
 private:
     unsigned indexM, stringIndexM;
     bool textualM;
+    wxMBConv* converterM;
 public:
     BlobColumnDef(const wxString& name, bool readOnly, bool nullable,
         unsigned stringIndex, unsigned blobIndex, bool textual);
+    void reset(DataGridRowBuffer* buffer);
     virtual unsigned getIndex();
     virtual wxString getAsString(DataGridRowBuffer* buffer);
     virtual unsigned getBufferSize();
@@ -1062,6 +1198,11 @@ BlobColumnDef::BlobColumnDef(const wxString& name, bool readOnly,
     readOnlyM = true;   // TODO: uncomment this when we make BlobDialog
 }
 //-----------------------------------------------------------------------------
+void BlobColumnDef::reset(DataGridRowBuffer* buffer)
+{
+    buffer->setString(stringIndexM, wxEmptyString);
+}
+//-----------------------------------------------------------------------------
 unsigned BlobColumnDef::getIndex()
 {
     return indexM;
@@ -1076,7 +1217,7 @@ wxString BlobColumnDef::getAsString(DataGridRowBuffer* buffer)
         return wxT("[BINARY]");
 
     int kb = 1024 * config().get(wxT("DataGridFetchBlobAmount"), 1);
-    wxString result;
+    std::string result;
     IBPP::Blob *b0 = buffer->getBlob(indexM);
     if (!b0)
         return wxT("");
@@ -1096,7 +1237,7 @@ wxString BlobColumnDef::getAsString(DataGridRowBuffer* buffer)
         if (textualM)
         {
             std::string s(buffer, size);
-            result += std2wx(s);    // TODO: wxMBConv needed
+            result += s;    // we don't convert here due to incomplete strings
         }
         else    // binary (show as hexadecimal)
         {
@@ -1107,18 +1248,27 @@ wxString BlobColumnDef::getAsString(DataGridRowBuffer* buffer)
                     last = size - i;
                 for (int j=0; j<last; j++)
                 {
-                    result += wxString::Format(wxT("%02X"),
-                        (unsigned char)(buffer[i+j]));
+                    result += wx2std(wxString::Format(wxT("%02X"),
+                        (unsigned char)(buffer[i+j])));
                 }
-                result += wxT(" ");
+                result += " ";
                 if (((i + 8) % 32) == 0)
-                    result += wxT("\n");
+                    result += "\n";
             }
         }
     }
     b->Close();
-    buffer->setString(stringIndexM, result); // store for future calls
-    return result;
+    wxString wxs(std2wx(result, converterM));
+    if (kb <= 0)    // there was more data to fetch
+    {               // incomplete strings might not get translated properly
+        while (wxs.IsEmpty() && result.length() > 0)
+        {
+            result.erase(result.length()-1, 1); // remove last byte
+            wxs = std2wx(result, converterM);   // try converting again
+        }
+    }
+    buffer->setString(stringIndexM, wxs);
+    return wxs;
 }
 //-----------------------------------------------------------------------------
 void BlobColumnDef::setFromString(DataGridRowBuffer* /*buffer*/,
@@ -1135,13 +1285,14 @@ unsigned BlobColumnDef::getBufferSize()
 }
 //-----------------------------------------------------------------------------
 void BlobColumnDef::setValue(DataGridRowBuffer* buffer, unsigned col,
-    const IBPP::Statement& statement, wxMBConv* /*converter*/)
+    const IBPP::Statement& statement, wxMBConv* converter)
 {
     wxASSERT(buffer);
     IBPP::Blob b = IBPP::BlobFactory(statement->DatabasePtr(),
         statement->TransactionPtr());
     statement->Get(col, b);
     buffer->setBlob(indexM, b);
+    converterM = converter; // store for later when we fetch the data
 }
 //-----------------------------------------------------------------------------
 // StringColumnDef class
@@ -1292,13 +1443,6 @@ void DataGridRows::clear()
     bufferSizeM = 0;
 }
 //-----------------------------------------------------------------------------
-wxString escapeQuotes(const wxString& input)
-{
-    wxString sCopy(input);
-    sCopy.Replace(wxT("'"), wxT("''"));
-    return sCopy;
-}
-//-----------------------------------------------------------------------------
 bool DataGridRows::canRemoveRow(size_t row)
 {
     if (row >= buffersM.size())
@@ -1374,8 +1518,9 @@ bool DataGridRows::removeRows(size_t from, size_t count, wxString& stm)
             stm += wxTextBuffer::GetEOL();
         wxString s = wxT("DELETE FROM ")
             + Identifier((*deleteFromM).first).getQuoted() + wxT(" WHERE ");
-        addWhereAndExecute((*deleteFromM).second, s, (*deleteFromM).first,
-            buffersM[from+pos]);
+        IBPP::Statement st = addWhere((*deleteFromM).second, s,
+            (*deleteFromM).first, buffersM[from+pos]);
+        st->Execute();
         stm += s + wxT(";");
     }
 
@@ -1701,7 +1846,7 @@ bool DataGridRows::isFieldNA(unsigned row, unsigned col)
     return buffersM[row]->isFieldNA(col);
 }
 //-----------------------------------------------------------------------------
-void DataGridRows::addWhereAndExecute(UniqueConstraint* uq, wxString& stm,
+IBPP::Statement DataGridRows::addWhere(UniqueConstraint* uq, wxString& stm,
     const wxString& table, DataGridRowBuffer *buffer)
 {
     bool dbkey = false;
@@ -1725,8 +1870,7 @@ void DataGridRows::addWhereAndExecute(UniqueConstraint* uq, wxString& stm,
                 if (ci != uq->begin())
                     stm += wxT(" AND ");
                 stm += Identifier(cn).getQuoted() + wxT(" = '");
-                stm += escapeQuotes(
-                    columnDefsM[c2-1]->getAsFirebirdString(buffer));
+                stm += columnDefsM[c2-1]->getAsFirebirdString(buffer);
                 stm += wxT("'");
                 break;
             }
@@ -1755,11 +1899,94 @@ void DataGridRows::addWhereAndExecute(UniqueConstraint* uq, wxString& stm,
                     throw FRError(_("N/A value in DB_KEY column."));
                 IBPP::DBKey dbkey;
                 dbk->getDBKey(dbkey, buffer);
-                st->Set(1, dbkey);
+                // if updating BLOB, param = 2, else param = 1
+                st->Set(st->Parameters(), dbkey);
             }
         }
     }
-    st->Execute();
+    return st;
+}
+//-----------------------------------------------------------------------------
+bool DataGridRows::isBlobColumn(unsigned col)
+{
+    return (0 != dynamic_cast<BlobColumnDef *>(columnDefsM[col]));
+}
+//-----------------------------------------------------------------------------
+void DataGridRows::exportBlobFile(const wxString& filename, unsigned row,
+    unsigned col, ProgressIndicator *pi)
+{
+    wxFFile fl(filename, wxT("wb+"));
+    if (!fl.IsOpened())
+        throw FRError(_("Cannot open destination file."));
+    IBPP::Blob *b0 = buffersM[row]->getBlob(columnDefsM[col]->getIndex());
+    if (!b0)
+        throw FRError(_("BLOB data not valid"));
+    IBPP::Blob b = *b0;
+    b->Open();
+    int size;
+    b->Info(&size, 0, 0);
+    if (pi)
+        pi->initProgress(_("Saving..."), size);
+    while (!pi || !pi->isCanceled())
+    {
+        uint8_t buffer[32768];
+        int size = b->Read((void*)buffer, 32767);
+        if (size < 1)
+            break;
+        fl.Write(buffer, size);
+        if (pi)
+            pi->stepProgress(size);
+    }
+    fl.Close();
+    b->Close();
+}
+//-----------------------------------------------------------------------------
+void DataGridRows::importBlobFile(const wxString& filename, unsigned row,
+    unsigned col, ProgressIndicator *pi)
+{
+    wxFFile fl(filename, wxT("rb"));
+    if (!fl.IsOpened())
+        throw FRError(_("Cannot open BLOB file."));
+    if (pi)
+        pi->initProgress(_("Loading..."), fl.Length()); // wxFileOffset
+
+    wxString table(std2wx(statementM->ColumnTable(col+1)));
+    wxString stm = wxT("UPDATE ") + Identifier(table).getQuoted()
+        + wxT(" SET ")
+        + Identifier(std2wx(statementM->ColumnName(col+1))).getQuoted()
+        + wxT(" = ? WHERE ");
+    std::map<wxString, UniqueConstraint *>::iterator it =
+        statementTablesM.find(table);
+    if (it == statementTablesM.end() || (*it).second == 0)
+        throw FRError(_("Blob table not found."));
+    IBPP::Statement st = addWhere((*it).second, stm, table, buffersM[row]);
+    IBPP::Blob b = IBPP::BlobFactory(st->DatabasePtr(),
+        st->TransactionPtr());
+    b->Create();
+    uint8_t buffer[32768];
+    while (!fl.Eof())
+    {
+        size_t len = fl.Read(buffer, 32767);    // slow when not 32k
+        if (len < 1 || pi && pi->isCanceled())
+            break;
+        b->Write(buffer, len);
+        if (pi)
+            pi->stepProgress(len);
+    }
+    fl.Close();
+    b->Close();
+    if (pi && pi->isCanceled())
+        return;
+    st->Set(1, b);
+    st->Execute();  // we execute before updating internal storage
+
+    buffersM[row]->setBlob(columnDefsM[col]->getIndex(), b);
+    buffersM[row]->setFieldNull(col, false);
+    buffersM[row]->setFieldNA(col, false);
+    BlobColumnDef *bcd = dynamic_cast<BlobColumnDef *>(columnDefsM[col]);
+    if (!bcd)
+        throw FRError(_("Not a BLOB column."));
+    bcd->reset(buffersM[row]);  // reset cached blob data
 }
 //-----------------------------------------------------------------------------
 // returns the executed SQL statement
@@ -1807,8 +2034,8 @@ wxString DataGridRows::setFieldValue(unsigned row, unsigned col,
             stm += wxT(" = NULL WHERE ");
         else
         {
-            stm += wxT(" = '") + escapeQuotes(
-                columnDefsM[col]->getAsFirebirdString(buffersM[row]))
+            stm += wxT(" = '") +
+                columnDefsM[col]->getAsFirebirdString(buffersM[row])
                 + wxT("' WHERE ");
         }
 
@@ -1820,7 +2047,8 @@ wxString DataGridRows::setFieldValue(unsigned row, unsigned col,
         if (it == statementTablesM.end() || (*it).second == 0)
             throw FRError(_("This column should not be editable"));
 
-        addWhereAndExecute((*it).second, stm, table, oldRecord);
+        IBPP::Statement st = addWhere((*it).second, stm, table, oldRecord);
+        st->Execute();
         delete oldRecord;
         return stm;
     }
