@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: DataGridTable.cpp 1634 2008-02-03 18:43:54Z babuskov $
+  $Id: DataGridTable.cpp 1677 2008-04-07 21:01:06Z babuskov $
 
 */
 
@@ -748,6 +748,30 @@ void DataGridTable::setFetchAllRecords(bool fetchall)
     fetchAllRowsM = fetchall;
 }
 //-----------------------------------------------------------------------------
+void DataGridTable::importBlobFile(const wxString& filename, int row, int col,
+    ProgressIndicator *pi)
+{
+    rowsM.importBlobFile(filename, row, col, pi);
+    
+    // tell the grid it's done
+    if (GetView())
+    {
+        wxGridTableMessage msg(this, wxGRIDTABLE_REQUEST_VIEW_GET_VALUES);
+        GetView()->ProcessTableMessage(msg);
+    }
+}
+//-----------------------------------------------------------------------------
+void DataGridTable::exportBlobFile(const wxString& filename, int row, int col,
+    ProgressIndicator *pi)
+{
+    rowsM.exportBlobFile(filename, row, col, pi);
+}
+//-----------------------------------------------------------------------------
+bool DataGridTable::isBlobColumn(int col)
+{
+    return rowsM.isBlobColumn(col);
+}
+//-----------------------------------------------------------------------------
 void DataGridTable::SetValue(int row, int col, const wxString& value)
 {
     // We need explicit exception handling here since wxGrid gets
