@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: DataGridTable.h 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: DataGridTable.h 1677 2008-04-07 21:01:06Z babuskov $
 
 */
 
@@ -41,6 +41,7 @@ class Database;
 class DataGridCell;
 class ResultsetColumnDef;
 class DataGridRowBuffer;
+class ProgressIndicator;
 //-----------------------------------------------------------------------------
 BEGIN_DECLARE_EVENT_TYPES()
     // this event is sent after new rows have been fetched
@@ -94,13 +95,14 @@ public:
     bool isNullCell(int row, int col);
     bool isNumericColumn(int col);
     bool isReadonlyColumn(int col);
+    bool isBlobColumn(int col);
     bool needsMoreRowsFetched();
     void setFetchAllRecords(bool fetchall);
     bool canInsertRows();
     bool canRemoveRow(size_t row);
 
     void setNullFlag(bool isNull);
-
+        
     // methods of wxGridTableBase
     virtual void Clear();
     virtual wxGridCellAttr* GetAttr(int row, int col,
@@ -114,6 +116,12 @@ public:
     virtual bool IsEmptyCell(int row, int col);
     virtual void SetValue(int row, int col, const wxString& value);
     virtual bool DeleteRows(size_t pos, size_t numRows);
+
+    // BLOBs can be huge, so we don't use SetValue for that
+    void importBlobFile(const wxString& filename, int row, int col,
+        ProgressIndicator *pi = 0);
+    void exportBlobFile(const wxString& filename, int row, int col,
+        ProgressIndicator *pi = 0);
 };
 //-----------------------------------------------------------------------------
 #endif
