@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: CreateDDLVisitor.cpp 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: CreateDDLVisitor.cpp 1709 2008-07-06 19:11:06Z babuskov $
 
 */
 //-----------------------------------------------------------------------------
@@ -77,7 +77,7 @@ wxString CreateDDLVisitor::getPrefixSql() const
 //-----------------------------------------------------------------------------
 wxString CreateDDLVisitor::getSuffixSql() const
 {
-    return postSqlM;
+    return postSqlM + grantSqlM;
 }
 //-----------------------------------------------------------------------------
 // this one is not called from "outside", but from visit(Table) function
@@ -109,7 +109,7 @@ void CreateDDLVisitor::visitColumn(Column& c)
                 if (!db || db->getDatabaseCharset() != charset)
                     preSqlM << wxT(" CHARACTER SET ") << charset;
                 if (db && db->isDefaultCollation(charset, collate))
-                        collate.clear();    // don't show default collations
+                    collate.clear();    // don't show default collations
             }
         }
         else
@@ -214,7 +214,7 @@ void CreateDDLVisitor::visitDatabase(Database& d)
         return;
     }
 
-    sqlM = preSqlM + wxT("\n") + postSqlM;
+    sqlM = preSqlM + wxT("\n") + postSqlM + grantSqlM;
     if (progressIndicatorM)
     {
         progressIndicatorM->setProgressMessage(_("Extraction complete."));
@@ -378,7 +378,7 @@ void CreateDDLVisitor::visitProcedure(Procedure& p)
         for (std::vector<Privilege>::const_iterator ci = priv->begin();
             ci != priv->end(); ++ci)
         {
-            temp += (*ci).getSql() + wxT("\n");
+            grantSqlM += (*ci).getSql() + wxT("\n");
         }
     }
 
@@ -416,7 +416,7 @@ void CreateDDLVisitor::visitProcedure(Procedure& p)
 
     postSqlM << temp << wxT("\n");
     temp.Replace(wxT("ALTER"), wxT("CREATE"), false);   // just first
-    sqlM << temp;
+    sqlM << temp << grantSqlM;
 
     // create empty procedure body (for database DDL dump)
     temp = p.getAlterSql(false);    // false = only headers
@@ -435,7 +435,7 @@ void CreateDDLVisitor::visitRole(Role& r)
         for (std::vector<Privilege>::const_iterator ci = priv->begin();
             ci != priv->end(); ++ci)
         {
-            postSqlM += (*ci).getSql();
+            grantSqlM += (*ci).getSql();
         }
     }
     wxString description = r.getDescription();
@@ -448,7 +448,7 @@ void CreateDDLVisitor::visitRole(Role& r)
                  << description << wxT("'\nwhere RDB$ROLE_NAME = '")
                  << name << wxT("';\n");
     }
-    sqlM = preSqlM + wxT("\n") + postSqlM;
+    sqlM = preSqlM + wxT("\n") + postSqlM + grantSqlM;
 }
 //-----------------------------------------------------------------------------
 // used by visit(Table)
@@ -471,7 +471,11 @@ void addIndex(std::vector<Index> *ix, wxString& sql, ColumnConstraint *cc)
 //-----------------------------------------------------------------------------
 void CreateDDLVisitor::visitTable(Table& t)
 {
-    preSqlM += wxT("CREATE TABLE ") + t.getQuotedName();
+    int type = t.getRelationType();
+    preSqlM += wxT("CREATE ");
+    if (type == 4 || type == 5)
+        preSqlM += wxT("GLOBAL TEMPORARY ");
+    preSqlM += wxT("TABLE ") + t.getQuotedName();
     wxString external = t.getExternalPath();
     if (!external.IsEmpty())
     {
@@ -556,9 +560,12 @@ void CreateDDLVisitor::visitTable(Table& t)
     const std::vector<Privilege>* priv = t.getPrivileges();
     if (priv)
         for (std::vector<Privilege>::const_iterator ci = priv->begin(); ci != priv->end(); ++ci)
-            postSqlM += (*ci).getSql() + wxT("\n");
+            grantSqlM += (*ci).getSql() + wxT("\n");
 
-    preSqlM += wxT("\n);\n");
+    preSqlM += wxT("\n)");
+    if (type == 4)
+        preSqlM += wxT("\nON COMMIT PRESERVE ROWS");
+    preSqlM += wxT(";\n");
 
     wxString description = t.getDescription();
     if (!description.IsEmpty())
@@ -571,20 +578,21 @@ void CreateDDLVisitor::visitTable(Table& t)
                  << name << wxT("';\n");
     }
 
-    sqlM = preSqlM + wxT("\n") + postSqlM;
+    sqlM = preSqlM + wxT("\n") + postSqlM + grantSqlM;
 }
 //-----------------------------------------------------------------------------
 void CreateDDLVisitor::visitTrigger(Trigger& t)
 {
     wxString object, type;
-    bool active;
+    bool active, db;
     int position;
-    t.getTriggerInfo(object, active, position, type);
+    t.getTriggerInfo(object, active, position, type, db);
     wxString source = t.getSource();
-    wxString relation = t.getRelation();
+    wxString relation = t.getTriggerRelation();
 
-    preSqlM << wxT("SET TERM ^ ;\nCREATE TRIGGER ") << t.getQuotedName()
-         << wxT(" FOR ") << relation;
+    preSqlM << wxT("SET TERM ^ ;\nCREATE TRIGGER ") << t.getQuotedName();
+    if (!db)
+        preSqlM << wxT(" FOR ") << relation;
     if (active)
         preSqlM << wxT(" ACTIVE\n");
     else
@@ -639,7 +647,7 @@ void CreateDDLVisitor::visitView(View& v)
         for (std::vector<Privilege>::const_iterator ci = priv->begin();
             ci != priv->end(); ++ci)
         {
-            postSqlM += (*ci).getSql() + wxT("\n");
+            grantSqlM += (*ci).getSql() + wxT("\n");
         }
     }
     wxString name(v.getName_());
@@ -675,6 +683,6 @@ void CreateDDLVisitor::visitView(View& v)
         }
     }
 
-    sqlM += preSqlM + wxT("\n") + postSqlM;
+    sqlM += preSqlM + wxT("\n") + postSqlM + grantSqlM;
 }
 //-----------------------------------------------------------------------------
