@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: constraints.cpp 1637 2008-02-17 23:14:02Z babuskov $
+  $Id: constraints.cpp 1705 2008-06-30 17:57:34Z babuskov $
 
 */
 
@@ -46,10 +46,15 @@
 //-----------------------------------------------------------------------------
 bool Constraint::isSystem() const
 {
-    return (getName_().StartsWith(wxT("RDB$")) 
+    return (getName_().StartsWith(wxT("RDB$"))
         || getName_().StartsWith(wxT("INTEG_")));
 }
 //-----------------------------------------------------------------------------
+const wxString Constraint::getTypeName() const
+{
+    return wxT("CONSTRAINT");
+}
+//-----------------------------------------------------------------------------
 wxString ColumnConstraint::getColumnList() const
 {
     wxString result;
