@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: database.cpp 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: database.cpp 1704 2008-06-29 21:21:18Z babuskov $
 
 */
 
@@ -43,6 +43,7 @@
 #include "config/Config.h"
 #include "config/DatabaseConfig.h"
 #include "core/FRError.h"
+#include "core/ProgressIndicator.h"
 #include "core/StringUtils.h"
 #include "engine/MetadataLoader.h"
 #include "MasterPassword.h"
@@ -192,7 +193,7 @@ Database::Database(const Database& rhs)
         functionsM(rhs.functionsM), generatorsM(rhs.generatorsM),
         proceduresM(rhs.proceduresM), rolesM(rhs.rolesM), tablesM(rhs.tablesM),
         sysTablesM(rhs.sysTablesM), triggersM(rhs.triggersM),
-        viewsM(rhs.viewsM), collationsM(rhs.collationsM), idM(rhs.idM), 
+        viewsM(rhs.viewsM), collationsM(rhs.collationsM), idM(rhs.idM),
         storeEncryptedPasswordM(rhs.storeEncryptedPasswordM)
 {
     if (rhs.connectionCredentialsM)
@@ -303,6 +304,29 @@ wxString Database::loadDomainNameForColumn(wxString table, wxString field)
     return std2wx(domain);
 }
 //-----------------------------------------------------------------------------
+void Database::getDatabaseTriggers(std::vector<Trigger *>& list)
+{
+    MetadataLoader* loader = getMetadataLoader();
+    MetadataLoaderTransaction tr(loader);
+
+    IBPP::Statement& st1 = loader->getStatement(
+        "select rdb$trigger_name from rdb$triggers "
+        "where rdb$relation_name is null "
+        "order by rdb$trigger_sequence"
+    );
+    st1->Execute();
+    while (st1->Fetch())
+    {
+        std::string name;
+        st1->Get(1, name);
+        name.erase(name.find_last_not_of(" ") + 1);
+        Trigger* t = dynamic_cast<Trigger*>(findByNameAndType(ntTrigger,
+            std2wx(name)));
+        if (t)
+            list.push_back(t);
+    }
+}
+//-----------------------------------------------------------------------------
 //! returns all collations for a given charset
 std::vector<wxString> Database::getCollations(wxString charset)
 {
@@ -550,7 +574,7 @@ Relation* Database::getRelationForTrigger(Trigger* trigger)
 {
     if (!trigger)
         return 0;
-    wxString relName = trigger->getRelation();
+    wxString relName = trigger->getTriggerRelation();
     if (relName.IsEmpty())
         return 0;
     return findRelation(Identifier(relName));
@@ -639,8 +663,8 @@ void Database::parseCommitedSql(const SqlStatement& stm)
     }
 
     // handle "CREATE INDEX", "ALTER INDEX" and "SET STATISTICS INDEX"
-    if (stm.getObjectType() == ntIndex && (
-        stm.actionIs(actCREATE) || stm.actionIs(actALTER) || stm.actionIs(actSET)))
+    if (stm.getObjectType() == ntIndex && ( stm.actionIs(actCREATE)
+        || stm.actionIs(actALTER) || stm.actionIs(actSET)))
     {
         wxString tableName = getTableForIndex(stm.getName());
         MetadataItem* m = findByNameAndType(ntTable, tableName);
@@ -649,7 +673,7 @@ void Database::parseCommitedSql(const SqlStatement& stm)
         return;
     }
 
-    // update all TABLEs and VIEWs on "DROP TRIGGER"
+    // update all TABLEs, VIEWs and DATABASE on "DROP TRIGGER"
     if (stm.actionIs(actDROP, ntTrigger))
     {
         MetadataCollection<Table>::iterator itt;
@@ -658,16 +682,19 @@ void Database::parseCommitedSql(const SqlStatement& stm)
         MetadataCollection<View>::iterator itv;
         for (itv = viewsM.begin(); itv != viewsM.end(); itv++)
             (*itv).notifyObservers();
+        notifyObservers();
     }
 
     if (stm.actionIs(actCREATE) || stm.actionIs(actDECLARE))
     {
-        if (addObject(stm.getObjectType(), stm.getName())) // inserts object into collection
+        if (addObject(stm.getObjectType(), stm.getName()))
             refreshByType(stm.getObjectType());
-        // when trigger created -> force relations to update their property pages
+        // when trigger created: force relations to update their property pages
         Relation *r = stm.getCreateTriggerRelation();
         if (r)
             r->notifyObservers();
+        else if (stm.getObjectType() == ntTrigger) // database trigger probably
+            notifyObservers();
         return;
     }
 
@@ -765,6 +792,8 @@ void Database::parseCommitedSql(const SqlStatement& stm)
                         Relation* r = getRelationForTrigger(tr);
                         if (r)
                             r->notifyObservers();
+                        else  // database trigger
+                            notifyObservers();
                     }
                     break;
                 }
