@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: database.h 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: database.h 1704 2008-06-29 21:21:18Z babuskov $
 
 */
 
@@ -32,7 +32,6 @@
 
 #include <ibpp.h>
 
-#include "core/ProgressIndicator.h"
 #include "metadata/collection.h"
 #include "metadata/domain.h"
 #include "metadata/exception.h"
@@ -44,10 +43,11 @@
 #include "metadata/table.h"
 #include "metadata/trigger.h"
 #include "metadata/view.h"
-#include "sql/SqlStatement.h"
 //-----------------------------------------------------------------------------
 class MetadataLoader;
+class ProgressIndicator;
 class Server;
+class SqlStatement;
 //-----------------------------------------------------------------------------
 class Credentials
 {
@@ -219,6 +219,9 @@ public:
     //! fill vector with result of sql statement
     void fillVector(std::vector<wxString>& list, wxString sql);
 
+    //! gets the database triggers (FB2.1+)
+    void getDatabaseTriggers(std::vector<Trigger *>& list);
+
     wxString getPath() const;
     wxString getDatabaseCharset() const;
     wxString getConnectionCharset() const;
