@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: domain.cpp 1638 2008-02-24 20:28:13Z babuskov $
+  $Id: domain.cpp 1738 2008-09-13 16:04:29Z babuskov $
 
 */
 
@@ -67,7 +67,7 @@ void Domain::loadInfo()
     IBPP::Statement& st1 = loader->getStatement(
         "select t.rdb$type, f.rdb$field_sub_type, f.rdb$field_length,"
         " f.rdb$field_precision, f.rdb$field_scale, c.rdb$character_set_name, "
-        " c.rdb$bytes_per_character, f.rdb$null_flag, f.rdb$default_source, "
+        " f.rdb$character_length, f.rdb$null_flag, f.rdb$default_source, "
         " l.rdb$collation_name, f.rdb$validation_source "
         " from rdb$fields f"
         " join rdb$types t on f.rdb$field_type=t.rdb$type"
@@ -89,14 +89,10 @@ void Domain::loadInfo()
         subtypeM = 0;
     else
         st1->Get(2, &subtypeM);
-    st1->Get(3, &lengthM);
-    // rdb$character_length does not work properly with computed columns
-    if (!st1->IsNull(7))
-    {
-        int bpc;
-        st1->Get(7, bpc);
-        lengthM /= bpc;
-    }
+    if (st1->IsNull(7))
+        st1->Get(3, &lengthM);
+    else
+        st1->Get(7, &lengthM);
     if (st1->IsNull(4))
         precisionM = 0;
     else
@@ -114,15 +110,19 @@ void Domain::loadInfo()
         charsetM = std2wx(charset).Strip();
     }
     isNotNullM = !st1->IsNull(8);
-    readBlob(st1, 9, defaultM);
 
     hasDefaultM = !st1->IsNull(9);
     if (hasDefaultM)
     {
+        readBlob(st1, 9, defaultM);
+
         // Some users reported two spaces before DEFAULT word in source
-        // Perhaps some other tools can put garbage here? Should we
-        // parse it as SQL to clean up comments, whitespace, etc?
-        defaultM.Trim(false).Remove(0, 8);
+        // Also, equals sign is also allowed in newer FB versions
+        // Trim(false) is trim-left
+        if (defaultM.Trim(false).StartsWith(wxT("DEFAULT")))
+            defaultM.Remove(0, 8);
+        else if (defaultM.StartsWith(wxT("="))) // "=" or "= "
+            defaultM.Remove(0, 1).Trim(false);
     }
 
     if (st1->IsNull(10))
@@ -169,7 +169,7 @@ wxString Domain::datatype2string(short datatype, short scale, short precision,
         return std2wx(retval.str());
     }
 
-    // INTEGERs (prec=0), DECIMAL(sub_type=2), NUMERIC(sub_type=1)
+    // INTEGER(prec=0), DECIMAL(sub_type=2), NUMERIC(sub_t=1), BIGINT(sub_t=0)
     if (datatype == 7 || datatype == 8 || datatype == 16)
     {
         if (scale == 0 && (datatype == 7 || datatype == 8))
@@ -181,6 +181,8 @@ wxString Domain::datatype2string(short datatype, short scale, short precision,
         }
         else
         {
+            if (subtype == 0)
+                return wxT("Bigint");
             retval << (subtype == 2 ? "Decimal(" : "Numeric(");
             if (precision <= 0 || precision > 18)
                 retval << 18;
