@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: generator.cpp 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: generator.cpp 1711 2008-07-06 19:12:24Z babuskov $
 
 */
 
@@ -99,6 +99,21 @@ wxString Generator::getPrintableName()
     return std2wx(ss.str());
 }
 //-----------------------------------------------------------------------------
+void Generator::loadDescription()
+{
+    MetadataItem::loadDescription(
+        wxT("select RDB$DESCRIPTION from RDB$GENERATORS ")
+        wxT("where RDB$GENERATOR_NAME = ?"));
+}
+//-----------------------------------------------------------------------------
+void Generator::saveDescription(wxString description)
+{
+    MetadataItem::saveDescription(
+        wxT("update RDB$GENERATORS set RDB$DESCRIPTION = ? ")
+        wxT("where RDB$GENERATOR_NAME = ?"),
+        description);
+}
+//-----------------------------------------------------------------------------
 wxString Generator::getCreateSqlTemplate() const
 {
     return  wxT("CREATE GENERATOR name;\n")
