@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: generator.h 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: generator.h 1711 2008-07-06 19:12:24Z babuskov $
 
 */
 
@@ -37,6 +37,9 @@ private:
     int64_t valueM;
     bool valueLoadedM;
     void setValue(int64_t value);
+protected:
+    virtual void loadDescription();
+    virtual void saveDescription(wxString description);
 public:
     Generator();
     // overrides MetadataItem::getCreateSqlTemplate()
