@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: parameter.cpp 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: parameter.cpp 1710 2008-07-06 19:11:43Z babuskov $
 
 */
 
@@ -46,8 +46,8 @@
 #include "metadata/MetadataItemVisitor.h"
 #include "metadata/parameter.h"
 //-----------------------------------------------------------------------------
-Parameter::Parameter(wxString source, int parameterType)
-    : Column()
+Parameter::Parameter(wxString source, int parameterType, int mechanism)
+    : Column(), parameterMechanismM(mechanism)
 {
     Column::Init(true, source, wxT(""), wxT(""), wxT(""), false);
     typeM = (parameterType == 0) ? ntParameterInput : ntParameterOutput;
@@ -61,10 +61,8 @@ Parameter::Parameter()
 //-----------------------------------------------------------------------------
 wxString Parameter::getPrintableName()
 {
-    wxString ret;
-    ret = (typeM == ntParameterInput ? wxT("in ") : wxT("out ")) + getName_();
-    Domain* d = getDomain();
-    return ret + wxT(" ") + ((d) ? d->getDatatypeAsString() : getSource());
+    return (typeM == ntParameterInput ? wxT("in ") : wxT("out "))
+        + Column::getPrintableName();
 }
 //-----------------------------------------------------------------------------
 bool Parameter::isOutputParameter() const
@@ -72,6 +70,11 @@ bool Parameter::isOutputParameter() const
     return typeM == ntParameterOutput;
 }
 //-----------------------------------------------------------------------------
+int Parameter::getMechanism() const
+{
+    return parameterMechanismM;
+}
+//-----------------------------------------------------------------------------
 void Parameter::loadDescription()
 {
     MetadataItem::loadDescription(
