@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: parameter.h 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: parameter.h 1710 2008-07-06 19:11:43Z babuskov $
 
 */
 
@@ -35,11 +35,14 @@
 class Parameter: public Column
 {
 protected:
+    int parameterMechanismM;    // -1 none, 0 domain, 1 type_of
+
     virtual void loadDescription();
     virtual void saveDescription(wxString description);
 public:
     Parameter();
-    Parameter(wxString source, int parameterType);
+    Parameter(wxString source, int parameterType, int mechanism);
+    int getMechanism() const;
     wxString getPrintableName();
     bool isOutputParameter() const;
     virtual void acceptVisitor(MetadataItemVisitor* visitor);
