@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: privilege.cpp 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: privilege.cpp 1687 2008-04-23 08:21:27Z babuskov $
 
 */
 
@@ -148,8 +148,8 @@ wxString Privilege::getSql(bool withGrantOption) const
     if (dynamic_cast<Procedure *>(parentObjectM))
         ret += wxT("PROCEDURE ");
     Identifier id(granteeM);
-    ret += parentObjectM->getQuotedName()
-        + wxT(" TO ") + granteeTypeToString(granteeTypeM) + id.getQuoted();
+    ret += parentObjectM->getQuotedName() + wxT(" TO ")
+        + granteeTypeToString(granteeTypeM) + wxT(" ") + id.getQuoted();
 
     if (withGrantOption)
         ret += wxT(" WITH GRANT OPTION");
