@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: procedure.cpp 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: procedure.cpp 1723 2008-07-23 12:04:56Z babuskov $
 
 */
 
@@ -71,6 +71,11 @@ Procedure::Procedure(const Procedure& rhs)
     parametersM.setParent(this);
 }
 //-----------------------------------------------------------------------------
+bool Procedure::isParameterInfoLoaded() const
+{
+    return parametersLoadedM;
+}
+//-----------------------------------------------------------------------------
 Parameter* Procedure::addParameter(Parameter &c)
 {
     if (!parametersLoadedM)
@@ -181,24 +186,33 @@ void Procedure::loadParameters()
     MetadataLoaderTransaction tr(loader);
     SubjectLocker lock(this);
 
-    IBPP::Statement st1 = loader->getStatement(
+    std::string sql(
         "select p.rdb$parameter_name, p.rdb$field_source, p.rdb$parameter_type"
-        " from rdb$procedure_parameters p"
-        " where p.rdb$PROCEDURE_name = ? "
-        " order by p.rdb$parameter_type, rdb$PARAMETER_number "
     );
+    DatabaseInfo *dbi = d->getInfo();
+    int ods = dbi->getODS();
+    int minor = dbi->getODSMinor();
+    if (ods > 11 || ods == 11 && minor > 0)
+        sql += ", RDB$PARAMETER_MECHANISM ";
+    else
+        sql += ", -1 ";
+    sql +=  " from rdb$procedure_parameters p"
+            " where p.rdb$PROCEDURE_name = ? "
+            " order by p.rdb$parameter_type, rdb$PARAMETER_number ";
+    IBPP::Statement st1 = loader->getStatement(sql);
     st1->Set(1, wx2std(getName_()));
     st1->Execute();
 
     while (st1->Fetch())
     {
         std::string column_name, source;
-        short partype;
+        short partype, mechanism = -1;
         st1->Get(1, column_name);
         st1->Get(2, source);
         st1->Get(3, &partype);
-
-        Parameter p(std2wx(source), partype);
+        if (!st1->IsNull(4))
+            st1->Get(4, mechanism);
+        Parameter p(std2wx(source), partype, mechanism);
         p.setName_(std2wx(column_name));
         Parameter* pp = parametersM.add(p);
         pp->setParent(this);
@@ -300,19 +314,46 @@ wxString Procedure::getAlterSql(bool full)
             parametersM.begin(); it != parametersM.end(); ++it)
         {
             wxString charset;
+            wxString param = (*it).getQuotedName() + wxT(" ");
+            wxString collate = (*it).getCollation();
             Domain* dm = (*it).getDomain();
             if (dm)
-                charset = dm->getCharset();
+            {
+                if (dm->isSystem()) // autogenerated domain -> use datatype
+                {
+                    param += dm->getDatatypeAsString();
+                    charset = dm->getCharset();
+                    if (!charset.IsEmpty())
+                    {
+                        if (charset != db->getDatabaseCharset())
+                            charset = wxT(" CHARACTER SET ") + charset;
+                        else
+                            charset = wxT("");
+                    }
+                    if (db->isDefaultCollation(charset, collate))
+                        collate.clear();    // don't show default collations
+                }
+                else
+                {
+                    if ((*it).getMechanism() == 1)
+                        param += wxT("TYPE OF ") + dm->getQuotedName();
+                    else
+                        param += dm->getQuotedName();
+                }
+            }
+            else
+                param += (*it).getSource();
+
+            if (!collate.IsEmpty())
+                charset += wxT(" COLLATE ") + collate;
+
             if ((*it).isOutputParameter())
             {
                 if (output.empty())
                     output += wxT("\nRETURNS (\n    ");
                 else
                     output += wxT(",\n    ");
-                output += (*it).getQuotedName() + wxT(" ") +
-                    ((dm) ? dm->getDatatypeAsString() : (*it).getSource());
-                if (!charset.IsEmpty() && charset != db->getDatabaseCharset())
-                    output += wxT(" CHARACTER SET ") + charset;
+                output += param + charset;
             }
             else
             {
@@ -320,12 +361,12 @@ wxString Procedure::getAlterSql(bool full)
                     input += wxT(" (\n    ");
                 else
                     input += wxT(",\n    ");
-                input += (*it).getQuotedName() + wxT(" ") +
-                    ((dm) ? dm->getDatatypeAsString() : (*it).getSource());
+                input += param;
+                //if (!(*it).isNullable(false))   // false = don't check domain
+                //    input += wxT(" NOT NULL");
                 if (dm && dm->hasDefault())
                     input += wxT(" DEFAULT ") + dm->getDefault();
-                if (!charset.IsEmpty() && charset != db->getDatabaseCharset())
-                    input += wxT(" CHARACTER SET ") + charset;
+                input += charset;
             }
         }
 
@@ -338,7 +379,7 @@ wxString Procedure::getAlterSql(bool full)
     if (full)
         sql += getSource();
     else
-        sql += wxT("BEGIN EXIT; END");
+        sql += wxT("BEGIN SUSPEND; END");
     sql += wxT("^\nSET TERM ; ^\n");
     return sql;
 }
