@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: relation.h 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: relation.h 1709 2008-07-06 19:11:06Z babuskov $
 
 */
 
@@ -41,6 +41,11 @@
 class Relation: public MetadataItem
 {
 protected:
+    int relationTypeM;
+    wxString ownerM;
+    bool relationInfoLoadedM;
+    void loadInfo();
+
     void getDependentChecks(std::vector<CheckConstraint>& checks);
     void getDependentViews(std::vector<Relation*>& views);
 
@@ -57,6 +62,7 @@ public:
     virtual void unlockChildren();
 
     wxString getOwner();
+    int getRelationType();
 
     void checkAndLoadColumns();
     virtual void loadColumns();
@@ -71,7 +77,7 @@ public:
     std::vector<Privilege>* getPrivileges();
     bool getChildren(std::vector<MetadataItem *>& temp);
     void getTriggers(std::vector<Trigger *>& list,
-        Trigger::firingTimeType beforeOrAfter);
+        Trigger::fireTimeType beforeOrAfter);
 };
 //-----------------------------------------------------------------------------
 #endif // FR_RELATION_H
