@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: trigger.h 1631 2008-02-02 22:25:30Z mapopa $
+  $Id: trigger.h 1704 2008-06-29 21:21:18Z babuskov $
 
 */
 
@@ -35,6 +35,7 @@ class Trigger: public MetadataItem
 {
 private:
     bool infoIsLoadedM;
+    bool isDatabaseTriggerM;
     wxString objectM;
     bool activeM;
     int positionM;
@@ -45,15 +46,16 @@ protected:
 public:
     Trigger();
 
-    enum firingTimeType { afterTrigger, beforeTrigger };
+    enum fireTimeType { afterTrigger, beforeTrigger, databaseTrigger };
     wxString getCreateSqlTemplate() const;  // overrides MetadataItem::getCreateSqlTemplate()
 
     void loadInfo(bool force = false);
-    void getTriggerInfo(wxString& object, bool& active, int& position, wxString& type);
+    void getTriggerInfo(wxString& object, bool& active, int& position,
+        wxString& type, bool& isDatabaseTrigger);
     wxString getSource() const;
-    wxString getRelation();
+    wxString getTriggerRelation();
     static wxString getTriggerType(int flags);
-    firingTimeType getFiringTime();
+    fireTimeType getFiringTime();
     wxString getAlterSql();
 
     virtual const wxString getTypeName() const;
