@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: IncompleteStatement.cpp 1640 2008-02-24 21:10:18Z babuskov $
+  $Id: IncompleteStatement.cpp 1704 2008-06-29 21:21:18Z babuskov $
 
 */
 
@@ -182,7 +182,9 @@ Relation *IncompleteStatement::getAlterTriggerRelation(const wxString& sql)
             ntTrigger, id.get()));
         if (!t)
             return 0;
-        wxString relName = t->getRelation();
+        wxString relName = t->getTriggerRelation();
+        if (relName.IsEmpty())  // database trigger
+            return 0;
         r = dynamic_cast<Relation *>(databaseM->findByNameAndType(ntTable,
             relName));
         if (!r)
