@@ -21,7 +21,7 @@
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
-  $Id: SqlStatement.cpp 1640 2008-02-24 21:10:18Z babuskov $
+  $Id: SqlStatement.cpp 1735 2008-09-12 22:46:36Z babuskov $
 
 */
 
@@ -108,11 +108,11 @@ SqlStatement::SqlStatement(const wxString& sql, Database *db, const wxString&
     if (nameM.get().IsEmpty()) // non-reserved keyword used as identifier?
     {
         nameM.setFromSql(tokenStringsM[2]); // we take a lucky guess
-        identifierTokenIndexM = 2;          // ex.: CREATE DOMAIN CASCADE 
+        identifierTokenIndexM = 2;          // ex.: CREATE DOMAIN CASCADE
     }
     else
         identifierTokenIndexM = tokensM.size() - 1;
-    
+
     size_t typeTokenIndex = 1;
 
     // get action
@@ -355,7 +355,7 @@ SqlStatement::SqlStatement(const wxString& sql, Database *db, const wxString&
 
                 stt = tokensM[fieldNameIndex + 2];
                 isDatatypeM = (stt == kwCHAR || stt == kwVARCHAR
-                    || stt == kwINTEGER || stt == kwSMALLINT
+                    || stt == kwINTEGER || stt == kwSMALLINT || stt == kwBIGINT
                     || stt == kwDECIMAL || stt == kwNUMERIC
                     || stt == kwDATE || stt == kwTIME || stt == kwTIMESTAMP
                     || stt == kwFLOAT || stt == kwBLOB)
